@@ -24,7 +24,6 @@
 #include <glib-unix.h>
 #include <gio/gio.h>
 
-#include <grp.h>
 #include <pwd.h>
 #include <errno.h>
 
@@ -64,12 +63,6 @@ gboolean       gdm_find_display_session (GPid        pid,
 gboolean       gdm_get_pwent_for_name    (const char     *name,
                                           struct passwd **pwentp);
 
-gboolean       gdm_get_pwent_for_uid     (uid_t           uid,
-                                          struct passwd **pwentp);
-
-gboolean       gdm_get_grent_for_name    (const char     *name,
-                                          struct group  **grentp);
-
 gboolean       gdm_clear_close_on_exec_flag (int fd);
 
 char          *gdm_generate_random_bytes (gsize          size,
@@ -104,10 +97,6 @@ gboolean      gdm_activate_session_by_id  (GDBusConnection *connection,
                                            const char      *seat_id,
                                            const char      *session_id);
 
-gboolean gdm_terminate_session_by_id      (GDBusConnection *connection,
-                                           GCancellable    *cancellable,
-                                           const char      *session_id);
-
 void          gdm_load_env_d              (GdmLoadEnvVarFunc load_env_func,
                                            GdmExpandVarFunc  expand_func,
                                            gpointer          user_data);
